# -*- coding: utf-8 -*-
RESERVED_WORDS = [
    'aes128',
    'aes256',
    'all',
    'allowoverwrite',
    'analyse',
    'analyze',
    'and',
    'any',
    'array',
    'as',
    'asc',
    'authorization',
    'backup',
    'between',
    'binary',
    'blanksasnull',
    'both',
    'bytedict',
    'bzip2',
    'case',
    'cast',
    'check',
    'collate',
    'column',
    'constraint',
    'create',
    'credentials',
    'cross',
    'current_date',
    'current_time',
    'current_timestamp',
    'current_user',
    'current_user_id',
    'default',
    'deferrable',
    'deflate',
    'defrag',
    'delta',
    'delta32k',
    'desc',
    'disable',
    'distinct',
    'do',
    'else',
    'emptyasnull',
    'enable',
    'encode',
    'encrypt     ',
    'encryption',
    'end',
    'except',
    'explicit',
    'false',
    'for',
    'foreign',
    'freeze',
    'from',
    'full',
    'globaldict256',
    'globaldict64k',
    'grant',
    'group',
    'gzip',
    'having',
    'identity',
    'ignore',
    'ilike',
    'in',
    'initially',
    'inner',
    'intersect',
    'into',
    'is',
    'isnull',
    'join',
    'leading',
    'left',
    'like',
    'limit',
    'localtime',
    'localtimestamp',
    'lun',
    'luns',
    'lzo',
    'lzop',
    'minus',
    'mostly13',
    'mostly32',
    'mostly8',
    'natural',
    'new',
    'not',
    'notnull',
    'null',
    'nulls',
    'off',
    'offline',
    'offset',
    'oid',
    'old',
    'on',
    'only',
    'open',
    'or',
    'order',
    'outer',
    'overlaps',
    'parallel',
    'partition',
    'percent',
    'permissions',
    'placing',
    'primary',
    'raw',
    'readratio',
    'recover',
    'references',
    'respect',
    'rejectlog',
    'resort',
    'restore',
    'right',
    'select',
    'session_user',
    'similar',
    'snapshot ',
    'some',
    'sysdate',
    'system',
    'table',
    'tag',
    'tdes',
    'text255',
    'text32k',
    'then',
    'timestamp',
    'to',
    'top',
    'trailing',
    'true',
    'truncatecolumns',
    'union',
    'unique',
    'user',
    'using',
    'verbose',
    'wallet',
    'when',
    'where',
    'with',
    'without',
]

TOCSV_KWARGS = [
    'path_or_buf', 
    'sep', 
    'na_rep', 
    'float_format', 
    'columns', 
    'header', 
    # 'index', 
    'index_label', 
    'mode', 
    'encoding', 
    'compression', 
    'quoting', 
    'quotechar', 
    'line_terminator', 
    'chunksize', 
    'tupleize_cols', 
    'date_format', 
    'doublequote', 
    'escapechar', 
    'decimal',
]

COPY_KWARGS = [
    'delimiter',
    'ignoreheader',
    'quote_character',
    'dateformat',
    'timeformat',
    'region',
]

TYPES_MAP = {
    'O': 'varchar(256)',
    'int64': 'bigint',
    'float64': 'double precision',
}